@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (C) 2009-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2009 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -17,12 +17,15 @@ import os.path
 import sys
 import traceback
 from typing import Any
-from collections.abc import Callable, Sequence
+from collections.abc import Callable
 
 from xpra.scripts.config import read_config, make_defaults_struct, validate_config, save_config
 from xpra.gtk.signals import register_os_signals
 from xpra.gtk.window import add_close_accel
-from xpra.gtk.widget import scaled_image, imagebutton, label, choose_file, modify_fg, color_parse
+from xpra.gtk.widget import (
+    scaled_image, imagebutton, label, choose_file, modify_fg,
+    set_widget_bg_color, set_widget_fg_color, red, black,
+)
 from xpra.gtk.pixbuf import get_icon_pixbuf
 from xpra.util.str_fn import csv, repr_ellipsized
 from xpra.os_util import WIN32, OSX, gi_import
@@ -57,7 +60,7 @@ MODE_QUIC = "quic"
 
 # what we save in the config file:
 SAVED_FIELDS = [
-    "username", "password", "host", "port", "mode", "ssh_port",
+    "username", "password", "host", "port", "mode", "ssh_port", "path",
     "encoding", "quality", "min-quality", "speed", "min-speed",
     "proxy_port", "proxy_username", "proxy_key", "proxy_password",
     "proxy_host",
@@ -72,6 +75,7 @@ LAUNCHER_OPTION_TYPES = {
     "username": str,
     "password": str,
     "mode": str,
+    "path": str,
     "autoconnect": bool,
     "ssh_port": int,
     "proxy_host": str,
@@ -86,6 +90,7 @@ LAUNCHER_DEFAULTS = {
     "username": get_username(),
     "password": "",
     "mode": MODE_TCP,  # tcp,ssh,..
+    "path": "",
     "autoconnect": False,
     "ssh_port": 22,
     "proxy_host": "",
@@ -95,12 +100,8 @@ LAUNCHER_DEFAULTS = {
     "proxy_key": "",
 }
 
-black = color_parse("black")
-red = color_parse("red")
-white = color_parse("white")
 
-
-def get_active_item_index(optionmenu):
+def get_active_item_index(optionmenu) -> int:
     menu = optionmenu.get_popup()
     for i, x in enumerate(menu.get_children()):
         if hasattr(x, "get_active") and x.get_active():
@@ -108,7 +109,7 @@ def get_active_item_index(optionmenu):
     return -1
 
 
-def set_history_from_active(optionmenu):
+def set_history_from_active(optionmenu) -> None:
     # Used for MenuButton combo:
     # sets the first active menu entry as the "history" value (the selected item)
     i = get_active_item_index(optionmenu)
@@ -129,7 +130,7 @@ def has_mdns() -> bool:
     return False
 
 
-def get_connection_modes():
+def get_connection_modes() -> list[str]:
     modes = [MODE_SSH, MODE_NESTED_SSH]
     try:
         import ssl
@@ -151,13 +152,13 @@ def get_connection_modes():
     return modes
 
 
-def image_button(text="", tooltip="", icon_pixbuf=None, clicked_cb=None):
+def image_button(text="", tooltip="", icon_pixbuf=None, clicked_cb=None) -> Gtk.Image:
     icon = Gtk.Image()
     icon.set_from_pixbuf(icon_pixbuf)
     return imagebutton(text, icon, tooltip, clicked_cb, icon_size=None)
 
 
-def button(tooltip, icon_name, callback):
+def button(tooltip, icon_name, callback) -> Gtk.Button:
     btn = Gtk.Button()
     theme = Gtk.IconTheme.get_default()
     try:
@@ -184,20 +185,37 @@ def button(tooltip, icon_name, callback):
         btn.add(image)
     btn.set_tooltip_text(tooltip)
 
-    def clicked(*_args):
+    def clicked(*_args) -> None:
         callback()
 
     btn.connect("clicked", clicked)
     return btn
 
 
+def get_launcher_validation() -> dict:
+    # TODO: since "mode" is not part of global options
+    # this validation should be injected from the launcher instead
+    def validate_in_list(x, options) -> str:
+        if x in options:
+            return ""
+        return "must be in " + csv(options)
+
+    modes = get_connection_modes()
+    return {"mode": lambda x: validate_in_list(x, modes)}
+
+
+def accel_close(*args) -> None:
+    log("accel_close%s", args)
+    Gtk.main_quit()
+
+
 class ApplicationWindow:
 
     def __init__(self):
         # Default connection options
         self.config = make_defaults_struct(extras_defaults=LAUNCHER_DEFAULTS,
                                            extras_types=LAUNCHER_OPTION_TYPES,
-                                           extras_validation=self.get_launcher_validation())
+                                           extras_validation=get_launcher_validation())
         self.parse_ssh()
         # TODO: the fixup does not belong here?
         from xpra.scripts.main import fixup_options
@@ -209,27 +227,16 @@ class ApplicationWindow:
         self.exit_code = None
         self.current_error = None
 
-    def parse_ssh(self):
+    def parse_ssh(self) -> None:
         ssh_cmd = parse_ssh_option(self.config.ssh)[0].strip().lower()
         self.is_putty = ssh_cmd.endswith("plink") or ssh_cmd.endswith("plink.exe")
         self.is_paramiko = ssh_cmd.startswith("paramiko")
 
-    def get_launcher_validation(self) -> dict:
-        # TODO: since "mode" is not part of global options
-        # this validation should be injected from the launcher instead
-        def validate_in_list(x, options):
-            if x in options:
-                return None
-            return "must be in " + csv(options)
-
-        modes = get_connection_modes()
-        return {"mode": lambda x: validate_in_list(x, modes)}
-
-    def create_window_with_config(self):
+    def create_window_with_config(self) -> None:
         self.do_create_window()
         self.update_gui_from_config()
 
-    def do_create_window(self):
+    def do_create_window(self) -> None:
         self.window = Gtk.Window()
         self.window.set_border_width(20)
         self.window.connect("delete-event", self.destroy)
@@ -238,7 +245,7 @@ class ApplicationWindow:
         self.window.set_position(Gtk.WindowPosition.CENTER)
         with IgnoreWarningsContext():
             self.window.set_wmclass("xpra-launcher-gui", "Xpra-Launcher-GUI")
-        add_close_accel(self.window, self.accel_close)
+        add_close_accel(self.window, accel_close)
         icon = get_icon_pixbuf("connect.png")
         if icon:
             self.window.set_icon(icon)
@@ -248,13 +255,13 @@ class ApplicationWindow:
         hb.props.title = "Session Launcher"
         self.window.set_titlebar(hb)
 
-        def show_about(*args):
+        def show_about(*_args) -> None:
             about(parent=self.window)
 
         hb.add(hb_button("About", "help-about", show_about))
         self.bug_tool = None
 
-        def bug(*_args):
+        def bug(*_args) -> None:
             if self.bug_tool is None:
                 from xpra.gtk.dialogs.bug_report import BugReport
                 self.bug_tool = BugReport()
@@ -265,12 +272,12 @@ class ApplicationWindow:
         if has_mdns():
             self.mdns_gui = None
 
-            def mdns(*_args):
+            def mdns(*_args) -> None:
                 if self.mdns_gui is None:
                     from xpra.gtk.dialogs.mdns_gui import mdns_sessions
                     self.mdns_gui = mdns_sessions(self.config)
 
-                    def close_mdns():
+                    def close_mdns() -> None:
                         mdns_gui = self.mdns_gui
                         if mdns_gui:
                             self.mdns_gui = None
@@ -405,6 +412,11 @@ class ApplicationWindow:
         self.port_entry.connect("changed", self.validate)
         self.port_entry.connect("activate", self.connect_clicked)
         self.port_entry.set_tooltip_text("port/display")
+        self.path_entry = Gtk.Entry()
+        self.path_entry.set_max_length(128)
+        self.path_entry.set_width_chars(5)
+        self.path_entry.connect("activate", self.connect_clicked)
+        self.path_entry.set_tooltip_text("request path")
         hbox.pack_start(label("Server:"), False, False)
         hbox.pack_start(self.username_entry, True, True)
         hbox.pack_start(label("@"), False, False)
@@ -412,6 +424,8 @@ class ApplicationWindow:
         hbox.pack_start(self.ssh_port_entry, False, False)
         hbox.pack_start(label(":"), False, False)
         hbox.pack_start(self.port_entry, False, False)
+        hbox.pack_start(label("/"), False, False)
+        hbox.pack_start(self.path_entry, False, False)
         vbox.pack_start(hbox)
 
         # Password
@@ -480,11 +494,7 @@ class ApplicationWindow:
         self.window.vbox = vbox
         self.window.add(vbox)
 
-    def accel_close(self, *args):
-        log("accel_close%s", args)
-        Gtk.main_quit()
-
-    def validate(self, *args):
+    def validate(self, *args) -> list:
         mode = self.mode_combo.get_active_text().lower()
         ssh = mode == MODE_SSH
         sshtossh = mode == MODE_NESTED_SSH
@@ -545,7 +555,7 @@ class ApplicationWindow:
         errs.append((self.port_entry, port < 0 or port >= 2 ** 16, "invalid port number"))
         err_text = []
         for w, e, text in errs:
-            self.set_widget_bg_color(w, e)
+            set_widget_bg_color(w, e)
             if e:
                 err_text.append(text)
         log(f"validate({args}) err_text={err_text}, errs={errs}")
@@ -553,7 +563,7 @@ class ApplicationWindow:
         self.connect_btn.set_sensitive(len(err_text) == 0)
         return errs
 
-    def show(self):
+    def show(self) -> None:
         self.window.show()
         self.window.present()
         self.connect_btn.grab_focus()
@@ -562,7 +572,7 @@ class ApplicationWindow:
         Gtk.main()
         return self.exit_code or 0
 
-    def mode_changed(self, *_args):
+    def mode_changed(self, *_args) -> None:
         mode = self.mode_combo.get_active_text().lower()
         ssh = mode == MODE_SSH
         sshtossh = mode == MODE_NESTED_SSH
@@ -632,13 +642,18 @@ class ApplicationWindow:
             if sshtossh:
                 self.check_boxes_hbox.hide()
                 self.proxy_password_hbox.hide()
+        can_use_path = mode in ("ws", "wss")
+        if can_use_path:
+            self.path_entry.show()
+        else:
+            self.path_entry.hide()
         self.validate()
         if mode in (MODE_SSL, MODE_WSS, MODE_QUIC) or (mode == MODE_SSH and not WIN32):
             self.nostrict_host_check.show()
         else:
             self.nostrict_host_check.hide()
 
-    def reset_errors(self):
+    def reset_errors(self) -> None:
         self.set_sensitive(True)
         self.set_info_text("")
         for widget in (
@@ -646,31 +661,31 @@ class ApplicationWindow:
                 self.port_entry, self.proxy_password_entry, self.proxy_username_entry,
                 self.proxy_host_entry, self.proxy_port_entry, self.ssh_port_entry,
         ):
-            self.set_widget_fg_color(self.info, False)
-            self.set_widget_bg_color(widget, False)
+            set_widget_fg_color(self.info, False)
+            set_widget_bg_color(widget, False)
 
-    def set_info_text(self, text, is_error=False):
+    def set_info_text(self, text, is_error=False) -> None:
         if self.info:
-            def do_set_info():
+            def do_set_info() -> None:
                 self.info.set_text(text)
                 self.info.set_selectable(is_error)
-                self.set_widget_fg_color(self.info, is_error)
+                set_widget_fg_color(self.info, is_error)
 
             GLib.idle_add(do_set_info)
 
-    def set_sensitive(self, s):
+    def set_sensitive(self, s) -> None:
         GLib.idle_add(self.window.set_sensitive, s)
 
-    def choose_pkey_file(self, title, action, action_button, callback):
+    def choose_pkey_file(self, title, action, action_button, callback) -> None:
         file_filter = Gtk.FileFilter()
         file_filter.set_name("All Files")
         file_filter.add_pattern("*")
         choose_file(self.window, title, action, action_button, callback, file_filter)
 
-    def proxy_key_browse_clicked(self, *args):
+    def proxy_key_browse_clicked(self, *args) -> None:
         log("proxy_key_browse_clicked%s", args)
 
-        def do_choose(filename):
+        def do_choose(filename) -> None:
             # make sure the file extension is .ppk
             if os.path.splitext(filename)[-1] != ".ppk":
                 filename += ".ppk"
@@ -678,12 +693,12 @@ class ApplicationWindow:
 
         self.choose_pkey_file("Choose SSH private key", Gtk.FileChooserAction.OPEN, Gtk.STOCK_OPEN, do_choose)
 
-    def connect_clicked(self, *args):
+    def connect_clicked(self, *args) -> None:
         log(f"connect_clicked({args})")
         self.update_options_from_gui()
         self.do_connect()
 
-    def clean_client(self):
+    def clean_client(self) -> None:
         c = self.client
         if c:
             c.disconnect_and_quit = noop
@@ -692,7 +707,7 @@ class ApplicationWindow:
             c.exit = noop
             c.cleanup()
 
-    def handle_exception(self, e):
+    def handle_exception(self, e) -> None:
         log(f"handle_exception({e})")
         t = str(e)
         log("handle_exception: %s", traceback.format_exc())
@@ -700,7 +715,7 @@ class ApplicationWindow:
             # in debug mode, include the full stacktrace:
             t = traceback.format_exc()
 
-        def ui_handle_exception():
+        def ui_handle_exception() -> None:
             self.clean_client()
             self.set_sensitive(True)
             if not self.current_error:
@@ -711,14 +726,14 @@ class ApplicationWindow:
 
         GLib.idle_add(ui_handle_exception)
 
-    def do_connect(self):
+    def do_connect(self) -> None:
         try:
             self.connect_builtin()
         except Exception as e:
             log.error("Error: cannot connect", exc_info=True)
             self.handle_exception(e)
 
-    def connect_builtin(self):
+    def connect_builtin(self) -> None:
         # cooked vars used by connect_to
         username = self.config.username
         params = {
@@ -730,7 +745,7 @@ class ApplicationWindow:
                 params["socket_dir"] = self.config.socket_dir
             params["remote_xpra"] = self.config.remote_xpra
             params["proxy_command"] = ["_proxy"]
-            if self.config.port and self.config.port > 0:
+            if self.config.port >= 0:
                 params["display"] = f":{self.config.port}"
                 params["display_as_args"] = [params["display"]]
             else:
@@ -789,6 +804,8 @@ class ApplicationWindow:
             params["local"] = is_local(self.config.host)
             params["port"] = int(self.config.port)
             params["display_name"] = f"{self.config.mode}://{self.config.host}:{self.config.port}"
+            if self.config.mode in (MODE_WS, MODE_WSS):
+                params["path"] = self.config.path
             # always populate ssl so we can auto-upgrade:
             params["ssl-options"] = get_ssl_options(params, self.config, [])
             if self.config.mode in (MODE_SSL, MODE_WSS, MODE_QUIC) and self.nostrict_host_check.get_active():
@@ -804,32 +821,29 @@ class ApplicationWindow:
         configure_network(self.config)
         self.start_client(params)
 
-    def start_client(self, display_desc):
-        def raise_exception(*args):
-            raise RuntimeError(*args)
-
+    def start_client(self, display_desc: dict) -> None:
         bypass_no_gtk()
-        self.client = make_client(raise_exception, self.config)
+        self.client = make_client(self.config)
         self.client.show_progress(30, "client configuration")
         self.client.init(self.config)
         self.client.show_progress(40, "loading user interface")
         self.client.init_ui(self.config)
         self.client.username = display_desc.get("username")
 
-        def handshake_complete(*_args):
+        def handshake_complete(*_args) -> None:
             self.client.show_progress(100, "Session connected")
 
         self.client.after_handshake(handshake_complete)
         self.set_info_text("Connecting...")
         start_thread(self.do_connect_builtin, "connect", daemon=True, args=(display_desc,))
 
-    def ssh_failed(self, message):
+    def ssh_failed(self, message) -> None:
         log(f"ssh_failed({message})")
         if not self.current_error:
             self.current_error = message
             self.set_info_text(message, True)
 
-    def do_connect_builtin(self, display_desc):
+    def do_connect_builtin(self, display_desc: dict) -> None:
         log(f"do_connect_builtin({display_desc})")
         self.exit_code = None
         self.current_error = None
@@ -846,33 +860,33 @@ class ApplicationWindow:
             self.handle_exception(e)
             return
         log("connect_to(..)=%s, hiding launcher window, starting client", conn)
-        GLib.idle_add(self.start_XpraClient, conn, display_desc)
+        GLib.idle_add(self.start_xpra_client, conn, display_desc)
 
-    def start_XpraClient(self, conn, display_desc):
+    def start_xpra_client(self, conn, display_desc: dict) -> None:
         try:
-            self.do_start_XpraClient(conn, display_desc)
+            self.do_start_xpra_client(conn, display_desc)
         except Exception as e:
             log.error("Error: failed to start client", exc_info=True)
             self.handle_exception(e)
 
-    def do_start_XpraClient(self, conn, display_desc):
+    def do_start_xpra_client(self, conn, display_desc: dict) -> None:
         log("do_start_XpraClient(%s, %s) client=%s", conn, display_desc, self.client)
         self.client.encoding = self.config.encoding
         self.client.display_desc = display_desc
-        self.client.setup_connection(conn)
+        self.client.make_protocol(conn)
         self.set_info_text("Network connection established")
         log("start_XpraClient() client initialized")
 
         if self.config.password:
             self.client.password = self.config.password
 
-        def do_quit(*args):
+        def do_quit(*args) -> None:
             log("do_quit%s", args)
             self.clean_client()
             self.destroy()
             Gtk.main_quit()
 
-        def handle_client_quit(exit_launcher=False):
+        def handle_client_quit(exit_launcher=False) -> None:
             w = self.window
             log("handle_quit(%s) window=%s", exit_launcher, w)
             self.clean_client()
@@ -884,7 +898,7 @@ class ApplicationWindow:
                     self.set_sensitive(True)
                     GLib.idle_add(w.show)
 
-        def reconnect(exit_code):
+        def reconnect(exit_code) -> bool:
             log("reconnect(%s) config reconnect=%s",
                 exit_str(exit_code), self.config.reconnect)
             if not self.config.reconnect or exit_code not in RETRY_EXIT_CODES:
@@ -894,7 +908,7 @@ class ApplicationWindow:
             GLib.timeout_add(100, self.start_client, display_desc)
             return True
 
-        def warn_and_quit_override(exit_code, warning):
+        def warn_and_quit_override(exit_code, warning) -> None:
             log("warn_and_quit_override(%s, %s)", exit_code, warning)
             password_warning = warning.find("invalid password") >= 0
             if password_warning:
@@ -909,7 +923,7 @@ class ApplicationWindow:
                 self.set_info_text(warning, err)
             handle_client_quit(not err)
 
-        def quit_override(exit_code):
+        def quit_override(exit_code) -> None:
             log("quit_override(%s)", exit_code)
             if reconnect(exit_code):
                 return
@@ -922,12 +936,12 @@ class ApplicationWindow:
         self.client.warn_and_quit = warn_and_quit_override
         self.client.quit = quit_override
 
-        def after_handshake():
+        def after_handshake() -> None:
             self.set_info_text("Handshake complete")
 
         self.client.after_handshake(after_handshake)
 
-        def first_ui_received(*_args):
+        def first_ui_received(*_args) -> None:
             self.set_info_text("Running")
             self.window.hide()
 
@@ -959,25 +973,19 @@ class ApplicationWindow:
             self.password_entry.modify_text(Gtk.StateType.NORMAL, red)
         self.password_entry.grab_focus()
 
-    def set_widget_bg_color(self, widget, is_error=False) -> None:
-        with IgnoreWarningsContext():
-            widget.modify_base(Gtk.StateType.NORMAL, red if is_error else white)
-
-    def set_widget_fg_color(self, widget, is_error=False) -> None:
-        modify_fg(widget, red if is_error else black)
-
     def update_options_from_gui(self) -> None:
-        def pint(vstr):
+        def pint(vstr, default=0) -> int:
             try:
                 return int(vstr)
             except ValueError:
-                return 0
+                return default
 
         self.config.host = self.host_entry.get_text()
         self.config.ssh_port = pint(self.ssh_port_entry.get_text())
-        self.config.port = pint(self.port_entry.get_text())
+        self.config.port = pint(self.port_entry.get_text(), -1)
         self.config.username = self.username_entry.get_text()
         self.config.password = self.password_entry.get_text()
+        self.config.path = self.path_entry.get_text()
         self.config.autoconnect = self.autoconnect.get_active()
 
         self.config.proxy_host = self.proxy_host_entry.get_text()
@@ -999,7 +1007,7 @@ class ApplicationWindow:
             (self.config.username, self.config.password, self.config.mode, self.config.encryption,
              self.config.host, self.config.port, self.config.ssh_port))
 
-    def update_gui_from_config(self):
+    def update_gui_from_config(self) -> None:
         mode = (self.config.mode or "").lower()
         active = 0
         for i, e in enumerate(get_connection_modes()):
@@ -1008,10 +1016,10 @@ class ApplicationWindow:
                 break
         self.mode_combo.set_active(active)
 
-        def get_port(vstr, default_port=""):
+        def get_port(vstr, default_port="") -> str:
             try:
                 iport = int(vstr)
-                if 0 < iport < 2 ** 16:
+                if 0 <= iport < 2 ** 16:
                     return str(iport)
             except ValueError:
                 pass
@@ -1039,6 +1047,7 @@ class ApplicationWindow:
         self.username_entry.set_text(username)
         self.password_entry.set_text(password)
         self.host_entry.set_text(self.config.host)
+        self.path_entry.set_text(self.config.path)
 
     def close_window(self, *_args) -> None:
         w = self.window
@@ -1054,7 +1063,7 @@ class ApplicationWindow:
         Gtk.main_quit()
         return False
 
-    def update_options_from_URL(self, url: str) -> None:
+    def update_options_from_url(self, url: str) -> None:
         from xpra.scripts.parsing import parse_URL
         address, props = parse_URL(url)
         pa = address.split("://")
@@ -1074,8 +1083,8 @@ class ApplicationWindow:
         self._apply_props(props)
 
     def update_options_from_file(self, filename: str) -> None:
-        log("update_options_from_file(%s)", filename)
         props = read_config(filename)
+        log("update_options_from_file(%s) props=%s", filename, props)
         self._apply_props(props)
 
     def _apply_props(self, props: dict[str, Any]) -> None:
@@ -1083,7 +1092,7 @@ class ApplicationWindow:
         # so try to load it from file, and define it if not present:
         options = validate_config(props,
                                   extras_types=LAUNCHER_OPTION_TYPES,
-                                  extras_validation=self.get_launcher_validation())
+                                  extras_validation=get_launcher_validation())
         for k, v in options.items():
             fn = k.replace("-", "_")
             setattr(self.config, fn, v)
@@ -1124,7 +1133,7 @@ def exception_dialog(title: str) -> None:
     md.format_secondary_text(traceback.format_exc())
     md.show_all()
 
-    def close_dialog(*_args):
+    def close_dialog(*_args) -> None:
         md.destroy()
         Gtk.main_quit()
 
@@ -1141,7 +1150,7 @@ def main(argv) -> int:
         return do_main(argv)
 
 
-def do_main(argv: Sequence[str]) -> int:
+def do_main(argv: list[str]) -> int:
     from xpra.util.system import SIGNAMES
     from xpra.scripts.main import InitExit, InitInfo
     from xpra.platform.gui import init as gui_init, ready as gui_ready
@@ -1155,8 +1164,7 @@ def do_main(argv: Sequence[str]) -> int:
         options, args = parse_cmdline(argv)
         debug = fixup_debug_option(options.debug)
         if debug:
-            for x in debug.split(","):
-                enable_debug_for(x)
+            enable_debug_for(*debug.split(","))
     except InitInfo as e:
         print(str(e))
         return 0
@@ -1173,7 +1181,7 @@ def do_main(argv: Sequence[str]) -> int:
     try:
         app = ApplicationWindow()
 
-        def handle_signal(signum):
+        def handle_signal(signum) -> None:
             app.show()
             client = app.client
             if client:
@@ -1181,7 +1189,6 @@ def do_main(argv: Sequence[str]) -> int:
             else:
                 Gtk.main_quit()
             GLib.timeout_add(1000, app.set_info_text, "got signal " + SIGNAMES.get(signum, signum))
-            GLib.timeout_add(1000, app.set_info_color, True)
 
         register_os_signals(handle_signal, "Client Launcher")
         has_file = len(args) == 1
@@ -1193,8 +1200,7 @@ def do_main(argv: Sequence[str]) -> int:
             configure_network(app.config)
         debug = fixup_debug_option(app.config.debug)
         if debug:
-            for x in debug.split(","):
-                enable_debug_for(x)
+            enable_debug_for(*debug.split(","))
         app.create_window_with_config()
     except Exception:
         exception_dialog("Error creating launcher form")
@@ -1213,7 +1219,7 @@ def do_main(argv: Sequence[str]) -> int:
                     force_focus()
                     app.show()
                 else:
-                    def open_file(filename):
+                    def open_file(filename: str) -> None:
                         log("open_file(%s)", filename)
                         app.update_options_from_file(filename)
                         # the compressors and packet encoders cannot be changed from the UI
@@ -1227,10 +1233,10 @@ def do_main(argv: Sequence[str]) -> int:
                             force_focus()
                             app.show()
 
-                    def open_URL(url):
+                    def open_URL(url: str) -> None:
                         log("open_URL(%s)", url)
                         app.__osx_open_signal = True
-                        app.update_options_from_URL(url)
+                        app.update_options_from_url(url)
                         # the compressors and packet encoders cannot be changed from the UI
                         # so apply them now:
                         configure_network(app.config)
