@@ -1,21 +1,24 @@
 # This file is part of Xpra.
-# Copyright (C) 2017-2023 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2017 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import sys
 import os.path
 from typing import Any
+from collections.abc import Sequence
 
 from xpra.exit_codes import ExitCode
-from xpra.os_util import WIN32
+from xpra.os_util import WIN32, POSIX, OSX, is_admin
+from xpra.util.io import load_binary_file, umask_context
 from xpra.scripts.config import InitExit, InitException, TRUE_OPTIONS
 from xpra.util.env import osexpand, envbool
 from xpra.util.parsing import parse_encoded_bin_data
-from xpra.util.str_fn import print_nested_dict, csv, ellipsizer, std
+from xpra.util.str_fn import print_nested_dict, csv, Ellipsizer, std
 
 SSL_RETRY = envbool("XPRA_SSL_RETRY", True)
 
-SSL_ATTRIBUTES = (
+SSL_ATTRIBUTES: Sequence[str] = (
     "cert", "key", "ca-certs", "ca-data",
     "protocol",
     "client-verify-mode", "server-verify-mode", "verify-flags",
@@ -23,7 +26,13 @@ SSL_ATTRIBUTES = (
     "options", "ciphers",
 )
 
+KEY_SIZE = 4096
+KEY_DAYS = 3650
+KEY_SUBJ = "/C=US/ST=Denial/L=Springfield/O=Dis/CN=localhost"
+
+KEY_FILENAME = "key.pem"
 CERT_FILENAME = "cert.pem"
+SSL_CERT_FILENAME = "ssl-cert.pem"
 
 logger = None
 
@@ -50,7 +59,7 @@ def get_ssl_attributes(opts, server_side: bool = True, overrides: dict | None =
     return args
 
 
-def find_ssl_cert(filename: str = "ssl-cert.pem") -> str:
+def find_ssl_cert(filename: str = SSL_CERT_FILENAME) -> str:
     ssllog = get_ssl_logger()
     # try to locate the cert file from known locations
     from xpra.platform.paths import get_ssl_cert_dirs  # pylint: disable=import-outside-toplevel
@@ -61,18 +70,18 @@ def find_ssl_cert(filename: str = "ssl-cert.pem") -> str:
         if not os.path.exists(p):
             ssllog(f"ssl cert dir {p!r} does not exist")
             continue
-        f = os.path.join(p, "ssl-cert.pem")
+        f = os.path.join(p, filename)
         if not os.path.exists(f):
             ssllog(f"ssl cert {f!r} does not exist")
             continue
         if not os.path.isfile(f):
             ssllog.warn(f"Warning: {f!r} is not a file")
             continue
-        if not os.access(p, os.R_OK):
+        if not os.access(f, os.R_OK):
             ssllog.info(f"SSL certificate file {f!r} is not accessible")
             continue
         ssllog(f"found ssl cert {f!r}")
-        return f
+        return os.path.abspath(f)
     return ""
 
 
@@ -122,7 +131,7 @@ class SSLVerifyFailure(InitExit):
         self.ssl_sock = ssl_sock
 
 
-def ssl_handshake(ssl_sock):
+def ssl_handshake(ssl_sock) -> None:
     ssllog = get_ssl_logger()
     try:
         ssl_sock.do_handshake(True)
@@ -134,7 +143,7 @@ def ssl_handshake(ssl_sock):
         import ssl
         ssleof_error = getattr(ssl, "SSLEOFError", None)
         if ssleof_error and isinstance(e, ssleof_error):
-            return None
+            return
         status = ExitCode.SSL_FAILURE
         ssl_cert_verification_error = getattr(ssl, "SSLCertVerificationError", None)
         if ssl_cert_verification_error and isinstance(e, ssl_cert_verification_error):
@@ -148,7 +157,6 @@ def ssl_handshake(ssl_sock):
             ssllog("host failed SSL verification: %s", msg)
             raise SSLVerifyFailure(status, msg, verify_code, ssl_sock) from None
         raise InitExit(status, f"SSL handshake failed: {e}") from None
-    return ssl_sock
 
 
 def get_ssl_verify_mode(verify_mode_str: str):
@@ -169,7 +177,9 @@ def get_ssl_wrap_socket_context(cert="", key="", key_password="", ca_certs="", c
                                 options: str = "ALL,NO_COMPRESSION", ciphers: str = "DEFAULT",
                                 server_side: bool = True):
     if server_side and not cert:
-        raise InitException("you must specify an 'ssl-cert' file to use ssl sockets")
+        cert = find_ssl_cert(SSL_CERT_FILENAME)
+        if not cert:
+            raise InitException("you must specify an 'ssl-cert' file to use ssl sockets")
     ssllog = get_ssl_logger()
     ssllog("get_ssl_wrap_socket_context%s", (
         cert, key, ca_certs, ca_data,
@@ -187,7 +197,7 @@ def get_ssl_wrap_socket_context(cert="", key="", key_password="", ca_certs="", c
     ssllog(" verify_mode for server_side=%s : %s", server_side, ssl_cert_reqs)
     # ca-certs:
     if ca_certs == "default":
-        ca_certs = None
+        ca_certs = ""
     elif ca_certs == "auto":
         ca_certs = find_ssl_cert("ca-cert.pem")
     ssllog(" ca-certs=%s", ca_certs)
@@ -202,9 +212,9 @@ def get_ssl_wrap_socket_context(cert="", key="", key_password="", ca_certs="", c
     ssllog(" protocol=%#x", proto)
     # ca_data may be hex encoded:
     ca_data = parse_encoded_bin_data(ca_data or "")
-    ssllog(" cadata=%s", ellipsizer(ca_data))
+    ssllog(" cadata=%s", Ellipsizer(ca_data))
 
-    kwargs = {
+    kwargs: dict[str, bool | str] = {
         "server_side": server_side,
         "do_handshake_on_connect": False,
         "suppress_ragged_eofs": True,
@@ -249,7 +259,8 @@ def get_ssl_wrap_socket_context(cert="", key="", key_password="", ca_certs="", c
         key_password = key_password or os.environ.get("XPRA_SSL_KEY_PASSWORD")
         ssllog("context.load_cert_chain%s", (cert or None, key or None, key_password))
         try:
-            context.load_cert_chain(certfile=cert, keyfile=key, password=key_password)
+            # we must pass a `None` value to ignore `keyfile`:
+            context.load_cert_chain(certfile=cert, keyfile=key or None, password=key_password)
         except ssl.SSLError as e:
             ssllog("load_cert_chain", exc_info=True)
             raise InitException(f"SSL error, failed to load certificate chain: {e}") from e
@@ -280,17 +291,8 @@ def get_ssl_wrap_socket_context(cert="", key="", key_password="", ca_certs="", c
             if not os.path.isfile(ca_certs):
                 raise InitException(f"{ca_certs!r} is not a valid ca file")
             context.load_verify_locations(cafile=ca_certs)
-        # handle cadata:
         if ca_data:
-            # PITA: because of a bug in the ssl module, we can't pass cadata,
-            # so we use a temporary file instead:
-            import tempfile
-            with tempfile.NamedTemporaryFile(prefix='cadata') as f:
-                ssllog(" loading cadata '%s'", ellipsizer(ca_data))
-                ssllog(" using temporary file '%s'", f.name)
-                f.file.write(ca_data)
-                f.file.flush()
-                context.load_verify_locations(cafile=f.name)
+            context.load_verify_locations(cadata=ca_data)
     elif check_hostname and not server_side:
         ssllog("cannot check hostname client side with verify mode %s", ssl_cert_reqs)
     return context, kwargs
@@ -323,13 +325,13 @@ def do_wrap_socket(tcp_socket, context, **kwargs):
     return ssl_sock
 
 
-def ssl_retry(e, ssl_ca_certs: str) -> dict[str, Any] | None:
+def ssl_retry(e, ssl_ca_certs: str) -> dict[str, Any]:
     ssllog = get_ssl_logger()
     ssllog("ssl_retry(%s, %s) SSL_RETRY=%s", e, ssl_ca_certs, SSL_RETRY)
     if not SSL_RETRY:
-        return None
+        return {}
     if not isinstance(e, SSLVerifyFailure):
-        return None
+        return {}
     # we may be able to ask the user if he wants to accept this certificate
     verify_code = e.verify_code
     ssl_sock = e.ssl_sock
@@ -344,10 +346,10 @@ def ssl_retry(e, ssl_ca_certs: str) -> dict[str, Any] | None:
             SSL_VERIFY_IP_MISMATCH, SSL_VERIFY_HOSTNAME_MISMATCH,
     ):
         ssllog("ssl_retry: %s not handled here", SSL_VERIFY_CODES.get(verify_code, verify_code))
-        return None
+        return {}
     if not server_hostname:
         ssllog("ssl_retry: no server hostname")
-        return None
+        return {}
     ssllog("ssl_retry: server_hostname=%s, ssl verify_code=%s (%i)",
            server_hostname, SSL_VERIFY_CODES.get(verify_code, verify_code), verify_code)
 
@@ -362,7 +364,7 @@ def ssl_retry(e, ssl_ca_certs: str) -> dict[str, Any] | None:
     if verify_code == SSL_VERIFY_SELF_SIGNED:
         if ssl_ca_certs not in ("", "default"):
             ssllog("self-signed cert does not match %r", ssl_ca_certs)
-            return None
+            return {}
         # perhaps we already have the certificate for this hostname
         cert_file = find_ssl_config_file(server_hostname, port, CERT_FILENAME)
         if cert_file:
@@ -377,18 +379,18 @@ def ssl_retry(e, ssl_ca_certs: str) -> dict[str, Any] | None:
             cert_data = None
         if not cert_data:
             ssllog.warn("Warning: failed to get server certificate from %s", addr)
-            return None
-        ssllog("downloaded ssl cert data for %s: %s", addr, ellipsizer(cert_data))
+            return {}
+        ssllog("downloaded ssl cert data for %s: %s", addr, Ellipsizer(cert_data))
         # ask the user if he wants to accept this certificate:
         title = "SSL Certificate Verification Failure"
         prompt = "Do you want to accept this certificate?"
         if not confirm((msg,), title, prompt):
-            return None
+            return {}
         filename = save_ssl_config_file(server_hostname, port,
                                         CERT_FILENAME, "certificate", cert_data.encode("latin1"))
         if not filename:
             ssllog.warn("Warning: failed to save certificate data")
-            return None
+            return {}
         options["ca-certs"] = filename
         save_ssl_options(server_hostname, port, options)
         return options
@@ -405,7 +407,7 @@ def ssl_retry(e, ssl_ca_certs: str) -> dict[str, Any] | None:
             options["check-hostname"] = False
             save_ssl_options(server_hostname, port, options)
             return options
-    return None
+    return {}
 
 
 def load_ssl_options(server_hostname: str, port: int) -> dict[str, Any]:
@@ -444,30 +446,38 @@ def save_ssl_options(server_hostname: str, port, options: dict) -> str:
     return f
 
 
-def find_ssl_config_file(server_hostname: str, port=443, filename="cert.pem"):
+def find_ssl_config_file(hostname: str, port=443, filename=CERT_FILENAME) -> str:
+    return do_find_ssl_config_file(hostname, port, filename) or do_find_ssl_config_file(hostname, 0, filename)
+
+
+def do_find_ssl_config_file(server_hostname: str, port=443, filename=CERT_FILENAME) -> str:
     ssllog = get_ssl_logger()
     from xpra.platform.paths import get_ssl_hosts_config_dirs
     dirs = get_ssl_hosts_config_dirs()
-    host_dirname = std(server_hostname, extras="-.:#_") + f"_{port}"
+    host_dirname = std(server_hostname, extras="-.:#_")
+    if port:
+        host_dirname += f"_{port}"
     host_dirs = [os.path.join(osexpand(d), host_dirname) for d in dirs]
     ssllog(f"looking for {filename!r} in {host_dirs}")
     for d in host_dirs:
         f = os.path.join(d, filename)
         if os.path.exists(f):
             ssllog(f"found {f}")
-            return f
-    return None
+            return os.path.abspath(f)
+    return ""
 
 
 def save_ssl_config_file(server_hostname: str, port=443,
-                         filename="cert.pem", fileinfo="certificate", filedata=b"") -> str:
+                         filename=CERT_FILENAME, fileinfo="certificate", filedata=b"") -> str:
     ssllog = get_ssl_logger()
     from xpra.platform.paths import get_ssl_hosts_config_dirs
     dirs = get_ssl_hosts_config_dirs()
-    host_dirname = std(server_hostname, extras="-.:#_") + f"_{port}"
+    host_dirname = std(server_hostname, extras="-.:#_")
+    if port:
+        host_dirname += f"_{port}"
     host_dirs = [os.path.join(osexpand(d), host_dirname) for d in dirs]
     ssllog(f"save_ssl_config_file%s dirs={dirs}, host_dirname={host_dirname}, host_dirs={host_dirs}",
-           (server_hostname, port, filename, fileinfo, ellipsizer(filedata)), )
+           (server_hostname, port, filename, fileinfo, Ellipsizer(filedata)), )
     # if there is an existing host config dir, try to use it:
     for d in [x for x in host_dirs if os.path.exists(x)]:
         f = os.path.join(d, filename)
@@ -501,3 +511,124 @@ def save_ssl_config_file(server_hostname: str, port=443,
         except OSError:
             ssllog(f"failed to save cert data to {d!r}", exc_info=True)
     return ""
+
+
+def gen_ssl_cert() -> tuple[str, str]:
+    log = get_ssl_logger()
+    keypath = find_ssl_cert(KEY_FILENAME)
+    certpath = find_ssl_cert(CERT_FILENAME)
+    if keypath and certpath:
+        log.info("found an existing SSL certificate:")
+        log.info(f" {keypath!r}")
+        log.info(f" {certpath!r}")
+        return keypath, certpath
+    from shutil import which
+    openssl = which("openssl") or os.environ.get("OPENSSL", "")
+    if not openssl:
+        raise InitExit(ExitCode.SSL_FAILURE, "cannot find openssl executable")
+    openssl_config = ""
+    creationflags = 0
+    if WIN32:
+        from xpra.platform.paths import get_app_dir
+        from subprocess import CREATE_NO_WINDOW
+        creationflags = CREATE_NO_WINDOW
+        openssl_config = os.path.join(get_app_dir(), "etc", "ssl", "openssl.cnf")
+    if is_admin():
+        # running as root, use global location:
+        if OSX:
+            xpra_dir = "/Library/Application Support/Xpra"
+        elif WIN32:
+            from xpra.platform.win32.paths import get_program_data_dir
+            xpra_dir = os.path.join(get_program_data_dir(), "Xpra")
+        else:
+            prefix = "" if sys.prefix == "/usr" else sys.prefix
+            xpra_dir = f"{prefix}/etc/xpra"
+        ssldir = f"{xpra_dir}/ssl"
+        if not os.path.exists(xpra_dir):
+            os.mkdir(ssldir, 0o777)
+        if not os.path.exists(ssldir):
+            os.mkdir(ssldir, 0o777)
+    else:
+        from xpra.platform.paths import get_ssl_cert_dirs
+        dirs = [d for d in get_ssl_cert_dirs() if not d.startswith("/etc") and not d.startswith("/usr") and d != "./"]
+        # use the first writeable one:
+        log(f"testing ssl dirs: {dirs}")
+        ssldir = ""
+        for sdir in dirs:
+            path = osexpand(sdir)
+            if os.path.exists(path) and os.path.isdir(path) and os.access(path, os.W_OK):
+                ssldir = path
+                log(f"found writeable ssl dir {ssldir!r}")
+                break
+        if not ssldir:
+            # we may have to create the parent directories:
+            log("no ssl dir found, trying to create one")
+            for sdir in dirs:
+                path = osexpand(sdir)
+                head = path
+                create_dirs = []
+                while head:
+                    head, tail = os.path.split(head)        #ie: "/home/user/.config/xpra", "ssl"
+                    log(f"{head=}, {tail=}")
+                    if tail:
+                        create_dirs.append(tail)
+                    else:
+                        break
+                    if tail.find("xpra") >= 0:
+                        # don't create directories above our one
+                        break
+                if create_dirs and head:
+                    while create_dirs:
+                        tail = create_dirs.pop()
+                        head = os.path.join(head, tail)
+                        try:
+                            os.mkdir(head, 0o777)
+                        except OSError as e:
+                            log(f"failed to create {head!r} for {path!r}: {e}")
+                            continue
+                    ssldir = path
+                    log(f"created {ssldir!r}")
+                    break
+    keypath = f"{ssldir}/{KEY_FILENAME}"
+    certpath = f"{ssldir}/{CERT_FILENAME}"
+    cmd = [
+        openssl,
+        "req", "-new",
+        "-newkey", f"rsa:{KEY_SIZE}",
+        "-days", f"{KEY_DAYS}",
+        "-nodes", "-x509",
+        "-subj", KEY_SUBJ,
+        "-keyout", keypath,
+        "-out", certpath,
+    ]
+    if openssl_config and os.path.exists(openssl_config):
+        cmd += ["-config", openssl_config]
+    log.info("generating a new SSL certificate:")
+    log.info(f" {keypath!r}")
+    log.info(f" {certpath!r}")
+    log(f"openssl command: {cmd}")
+    from subprocess import Popen
+    with umask_context(0o022):
+        with Popen(cmd, creationflags=creationflags) as p:
+            exit_code = p.wait()
+    if exit_code != 0:
+        raise InitExit(ExitCode.FAILURE, f"openssl command returned {exit_code}")
+    key = load_binary_file(keypath)
+    cert = load_binary_file(certpath)
+    sslcert = key+cert
+    sslcertpath = f"{ssldir}/{SSL_CERT_FILENAME}"
+    with open(sslcertpath, "wb") as f:
+        if POSIX:
+            os.fchmod(f.fileno(), 0o600)
+        f.write(sslcert)
+    return keypath, certpath
+
+
+def strip_cert(data: bytes) -> bytes:
+    BEGIN = b"-----BEGIN CERTIFICATE-----"
+    if data.find(BEGIN) >= 0:
+        data = BEGIN + data.split(BEGIN, 1)[1]
+    END = b"-----END CERTIFICATE-----"
+    if data.find(END) > 0:
+        data = data.split(END, 1)[0] + END + b"\n"
+    return data
