@@ -1,9 +1,10 @@
 # This file is part of Xpra.
-# Copyright (C) 2008, 2009 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
+# Copyright (C) 2011 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+from typing import Final
 from math import ceil, floor
 
 from xpra.os_util import gi_import
@@ -12,7 +13,7 @@ from xpra.util.env import envint, envbool
 from xpra.common import MAX_WINDOW_SIZE
 from xpra.gtk.gobject import one_arg_signal
 from xpra.gtk.error import XError, xsync, xswallow, xlog
-from xpra.x11.gtk_x11.prop import prop_set
+from xpra.x11.gtk.prop import prop_set
 from xpra.x11.prop_conv import MotifWMHints
 from xpra.x11.bindings.window import X11WindowBindings
 from xpra.x11.bindings.send_wm import send_wm_take_focus
@@ -20,7 +21,7 @@ from xpra.x11.common import Unmanageable
 from xpra.x11.models.size_hints_util import sanitize_size_hints
 from xpra.x11.models.base import BaseWindowModel, constants
 from xpra.x11.models.core import sanestr
-from xpra.x11.gtk3.bindings import add_event_receiver, remove_event_receiver
+from xpra.x11.gtk.bindings import add_event_receiver, remove_event_receiver
 from xpra.log import Logger
 
 log = Logger("x11", "window")
@@ -37,14 +38,14 @@ GLib = gi_import("GLib")
 
 X11Window = X11WindowBindings()
 
-CWX: int = constants["CWX"]
-CWY: int = constants["CWY"]
-CWWidth: int = constants["CWWidth"]
-CWHeight: int = constants["CWHeight"]
-CWBorderWidth: int = constants["CWBorderWidth"]
-CWSibling: int = constants["CWSibling"]
-CWStackMode: int = constants["CWStackMode"]
-CONFIGURE_GEOMETRY_MASK: int = CWX | CWY | CWWidth | CWHeight
+CWX: Final[int] = constants["CWX"]
+CWY: Final[int] = constants["CWY"]
+CWWidth: Final[int] = constants["CWWidth"]
+CWHeight: Final[int] = constants["CWHeight"]
+CWBorderWidth: Final[int] = constants["CWBorderWidth"]
+CWSibling: Final[int] = constants["CWSibling"]
+CWStackMode: Final[int] = constants["CWStackMode"]
+CONFIGURE_GEOMETRY_MASK: Final[int] = CWX | CWY | CWWidth | CWHeight
 CW_MASK_TO_NAME: dict[int, str] = {
     CWX: "X",
     CWY: "Y",
@@ -64,13 +65,15 @@ FORCE_XSETINPUTFOCUS = envbool("XPRA_FORCE_XSETINPUTFOCUS", False)
 VALIDATE_CONFIGURE_REQUEST = envbool("XPRA_VALIDATE_CONFIGURE_REQUEST", False)
 CLAMP_OVERLAP = envint("XPRA_WINDOW_CLAMP_OVERLAP", 20)
 assert CLAMP_OVERLAP >= 0
+SIZEHINTS_POSITION = envbool("XPRA_SIZEHINTS_POSITION", False)
+SIZEHINTS_SIZE = envbool("XPRA_SIZEHINTS_SIZE", False)
 
 
 def calc_constrained_size(width: int, height: int, hints) -> tuple[int, int]:
     if not hints:
         return width, height
 
-    def getintpair(key: str, dv1: int, dv2: int):
+    def getintpair(key: str, dv1: int, dv2: int) -> tuple[int, int]:
         v = hints.get(key)
         if v:
             try:
@@ -227,9 +230,9 @@ class WindowModel(BaseWindowModel):
     }
     __gsignals__ = dict(BaseWindowModel.__common_signals__)
     __gsignals__ |= {
-        "child-map-request-event": one_arg_signal,
-        "child-configure-request-event": one_arg_signal,
-        "xpra-destroy-event": one_arg_signal,
+        "x11-child-map-request-event": one_arg_signal,
+        "x11-child-configure-request-event": one_arg_signal,
+        "x11-destroy-event": one_arg_signal,
     }
 
     _property_names = BaseWindowModel._property_names + [
@@ -377,7 +380,7 @@ class WindowModel(BaseWindowModel):
         # by having WM_HINTS affect _NET_WM_STATE.  But this means that
         # WM_HINTS and _NET_WM_STATE handling become intertangled.
 
-        def set_if_unset(propname, value):
+        def set_if_unset(propname: str, value):
             # the property may not be initialized yet,
             # if that's the case then calling get_property throws an exception:
             try:
@@ -484,13 +487,13 @@ class WindowModel(BaseWindowModel):
     # X11 Events
     #########################################
 
-    def do_xpra_property_notify_event(self, event) -> None:
+    def do_x11_property_notify_event(self, event) -> None:
         if event.delivered_to == self.corral_xid:
             return
-        super().do_xpra_property_notify_event(event)
+        super().do_x11_property_notify_event(event)
 
     # noinspection PyMethodMayBeStatic
-    def do_child_map_request_event(self, event) -> None:
+    def do_x11_child_map_request_event(self, event) -> None:
         # If we get a MapRequest then it might mean that someone tried to map
         # this window multiple times in quick succession, before we actually
         # mapped it (so that several MapRequests ended up queued up; FSF Emacs
@@ -498,10 +501,10 @@ class WindowModel(BaseWindowModel):
         # the client is naughty and tried to map their window which is
         # currently not displayed.  In either case, we should just ignore the
         # request.
-        log("do_child_map_request_event(%s)", event)
+        log("do_x11_child_map_request_event(%s)", event)
 
-    def do_xpra_unmap_event(self, event) -> None:
-        log(f"do_xpra_unmap_event({event}) corral_xid={self.corral_xid:x}")
+    def do_x11_unmap_event(self, event) -> None:
+        log(f"do_x11_unmap_event({event}) corral_xid={self.corral_xid:x}")
         if not self.corral_xid or event.delivered_to == self.corral_xid:
             return
         if event.window != self.xid:
@@ -514,16 +517,16 @@ class WindowModel(BaseWindowModel):
         # Also, if we receive a *synthetic* UnmapNotify event, that always
         # means that the client has withdrawn the window (even if it was not
         # mapped in the first place) -- ICCCM section 4.1.4.
-        log("do_xpra_unmap_event(%s) client window unmapped, last_unmap_serial=%#x", event, self.last_unmap_serial)
+        log("do_x11_unmap_event(%s) client window unmapped, last_unmap_serial=%#x", event, self.last_unmap_serial)
         if event.send_event or self.serial_after_last_unmap(event.serial):
             self.unmanage()
 
-    def do_xpra_destroy_event(self, event) -> None:
-        log(f"do_xpra_destroy_event({event}) corral_xid={self.corral_xid:x}")
+    def do_x11_destroy_event(self, event) -> None:
+        log(f"do_x11_destroy_event({event}) corral_xid={self.corral_xid:x}")
         if not self.corral_xid or event.delivered_to == self.corral_xid:
             return
         assert event.window == self.xid
-        super().do_xpra_destroy_event(event)
+        super().do_x11_destroy_event(event)
 
     #########################################
     # Hooks for WM
@@ -535,16 +538,17 @@ class WindowModel(BaseWindowModel):
         if self.get_property("iconic"):
             self.set_property("iconic", False)
         self._update_client_geometry()
-        with xsync:
+        with xlog:
             if not X11Window.is_mapped(self.corral_xid):
                 X11Window.MapWindow(self.corral_xid)
 
     def hide(self) -> None:
         self._internal_set_property("shown", False)
-        with xsync:
+        with xlog:
             if X11Window.is_mapped(self.corral_xid):
                 X11Window.Unmap(self.corral_xid)
-            X11Window.Reparent(self.corral_xid, self.parking_window_xid, 0, 0)
+            if X11Window.getParent(self.corral_xid) != self.parking_window_xid:
+                X11Window.Reparent(self.corral_xid, self.parking_window_xid, 0, 0)
             X11Window.sendConfigureNotify(self.xid)
 
     def _update_client_geometry(self) -> None:
@@ -569,12 +573,11 @@ class WindowModel(BaseWindowModel):
         else:
             geometry = self.get_property("geometry")
             geomlog("_update_client_geometry: using current geometry=%s", geometry)
-        self._do_update_client_geometry(geometry)
+        self._do_update_client_geometry(*geometry)
 
-    def _do_update_client_geometry(self, geometry) -> None:
-        geomlog("_do_update_client_geometry(%s)", geometry)
+    def _do_update_client_geometry(self, x, y, allocated_w, allocated_h) -> None:
+        geomlog("_do_update_client_geometry(%s)", (x, y, allocated_w, allocated_h))
         hints = self.get_property("size-hints")
-        x, y, allocated_w, allocated_h = geometry
         w, h = self.calc_constrained_size(allocated_w, allocated_h, hints)
         geomlog("_do_update_client_geometry: size(%s)=%ix%i", hints, w, h)
         with xlog:
@@ -592,8 +595,8 @@ class WindowModel(BaseWindowModel):
                 X11Window.sendConfigureNotify(self.xid)
             self._updateprop("geometry", (x, y, w, h))
 
-    def do_xpra_configure_event(self, event) -> None:
-        geomlog("WindowModel.do_xpra_configure_event(%s) corral=%#x, client=%#x, managed=%s",
+    def do_x11_configure_event(self, event) -> None:
+        geomlog("WindowModel.do_x11_configure_event(%s) corral=%#x, client=%#x, managed=%s",
                 event, self.corral_xid, self.xid, self._managed)
         if not self._managed or not self.corral_xid or not self.xid:
             return
@@ -609,16 +612,16 @@ class WindowModel(BaseWindowModel):
             # workaround applications whose windows disappear from underneath us:
             with xsync:
                 if not X11Window.is_mapped(self.corral_xid):
-                    geomlog(f"WindowModel.do_xpra_configure_event: corral window {self.corral_xid:x} is not visible")
+                    geomlog(f"WindowModel.do_x11_configure_event: corral window {self.corral_xid:x} is not visible")
                     return
                 if not X11Window.is_mapped(self.xid):
-                    geomlog(f"WindowModel.do_xpra_configure_event: client window {self.xid:x} is not visible")
+                    geomlog(f"WindowModel.do_x11_configure_event: client window {self.xid:x} is not visible")
                     return
                 # event.border_width unused
                 self.configure_geometry(event.x, event.y, event.width, event.height)
                 self.update_children()
         except XError as e:
-            geomlog("do_xpra_configure_event(%s)", event, exc_info=True)
+            geomlog("do_x11_configure_event(%s)", event, exc_info=True)
             geomlog.warn("Warning: failed to resize corral window %#x", self.corral_xid)
             geomlog.warn(" %s", e)
 
@@ -644,9 +647,10 @@ class WindowModel(BaseWindowModel):
             children.append([xid] + list(geom))
         self._internal_set_property("children", children)
 
-    def configure_geometry(self, x: int, y: int, w: int, h: int) -> None:
+    def configure_geometry(self, x: int, y: int, w: int, h: int) -> bool:
         # the client window may have been resized or moved (generally programmatically)
         # so we may need to update the corral_window to match
+        geomlog("configure_geometry%s", (x, y, w, h))
         cow, coh = X11Window.getGeometry(self.corral_xid)[2:4]
         # size changes (and position if any):
         hints = self.get_property("size-hints")
@@ -657,7 +661,7 @@ class WindowModel(BaseWindowModel):
         geomlog("configure_geometry%s hints=%s, constrained size=%s, corral=%s, geometry=%s, resized=%s, moved=%s",
                 (x, y, w, h), hints, (w, h), (cow, coh), (cx, cy, cw, ch), resized, moved)
         if not (moved or resized):
-            return
+            return False
         if moved:
             self._internal_set_property("set-initial-position", True)
             self._internal_set_property("requested-position", (x, y))
@@ -675,11 +679,11 @@ class WindowModel(BaseWindowModel):
             X11Window.ResizeWindow(self.corral_xid, w, h)
             x = cx
             y = cy
-        self._updateprop("geometry", (x, y, w, h))
+        return self._updateprop("geometry", (x, y, w, h))
 
-    def do_child_configure_request_event(self, event) -> None:
+    def do_x11_child_configure_request_event(self, event) -> None:
         hints = self.get_property("size-hints")
-        geomlog("do_child_configure_request_event(%s) client=%#x, corral=%#x, value_mask=%s, size-hints=%s",
+        geomlog("do_x11_child_configure_request_event(%s) client=%#x, corral=%#x, value_mask=%s, size-hints=%s",
                 event, self.xid, self.corral_xid, configure_bits(event.value_mask), hints)
         if event.value_mask & CWStackMode:
             geomlog(" restack above=%s, detail=%s", event.above, event.detail)
@@ -723,7 +727,7 @@ class WindowModel(BaseWindowModel):
                 X11Window.sendConfigureNotify(self.xid)
             return
 
-        geomlog("do_child_configure_request_event updated requested geometry from %s to %s",
+        geomlog("do_x11_child_configure_request_event updated requested geometry from %s to %s",
                 (ox, oy, ow, oh), (x, y, w, h))
         mask = 0
         if ox != x:
@@ -868,11 +872,11 @@ class WindowModel(BaseWindowModel):
 
         # size and position are stored as properties, not as hints:
         pos = hints.pop("position", None)
-        if pos is not None:
+        if pos is not None and SIZEHINTS_POSITION:
             self._internal_set_property("requested-position", pos)
             self._internal_set_property("set-initial-position", True)
         size = hints.pop("size", None)
-        if size is not None:
+        if size is not None and SIZEHINTS_SIZE:
             self._internal_set_property("requested-size", size)
 
         # Don't send out notify and ConfigureNotify events when this property
@@ -901,16 +905,13 @@ class WindowModel(BaseWindowModel):
     def get_default_window_icon(self, _size: int = 48):
         return None
 
-    def get_wm_state(self, prop) -> bool:
+    def get_wm_state(self, prop: str) -> bool:
         state_names = self._state_properties.get(prop)
         assert state_names, f"invalid window state {prop}"
         log("get_wm_state(%s) state_names=%s", prop, state_names)
         # this is a virtual property for _NET_WM_STATE:
         # return True if any is set (only relevant for maximized)
-        for x in state_names:
-            if self._state_isset(x):
-                return True
-        return False
+        return any(self._state_isset(x) for x in state_names)
 
     ################################
     # Focus handling:
